@@ -26,7 +26,7 @@
 ;; details.
 ;;
 ;; A copy of the GNU General Public License is available at
-;; http://www.r-project.org/Licenses/
+;; https://www.r-project.org/Licenses/
 ;;
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -261,7 +261,7 @@ parenthetical grouping.")
        (member (current-word) kw-list)))
 
 
-(defconst gretl-font-lock-defaults
+(defconst gretl-font-lock-keywords
   (list
    ;; Fontify all builtin keywords.
    (cons (concat "\\<\\("
@@ -323,6 +323,7 @@ parenthetical grouping.")
 (defvar gretl-block-else-or-end-regexp
   (concat gretl-block-else-regexp "\\|" gretl-block-end-regexp))
 
+(defvar gretl-basic-offset 4)
 
 (defvar gretl-block-match-alist
   '(("loop" . ("endloop"))
@@ -433,14 +434,10 @@ end keywords as associated values.")
     (ess-local-process-name	  . nil)
     ;;(ess-keep-dump-files	    . 'ask)
     (ess-mode-syntax-table	  . gretl-syntax-table)
-  ;;  (add-log-current-defun-header-regexp . "^.*function[ \t]*\\([^ \t(]*\\)[ \t]*(")
-    (font-lock-defaults		  . '(gretl-font-lock-defaults))
+    ;;  (add-log-current-defun-header-regexp . "^.*function[ \t]*\\([^ \t(]*\\)[ \t]*(")
     )
   "General options for gretl source files.")
 
-(autoload 'inferior-ess "ess-inf" "Run an ESS process.")
-(autoload 'ess-mode     "ess-mode" "Edit an ESS process.")
-
 
 ;; (defun gretl-send-string-function (process string visibly)
 ;;   (let ((gretl-process (get-process "gretlcli")))
@@ -494,8 +491,8 @@ end keywords as associated values.")
     (inferior-ess-prompt		. "\\? ")
     (ess-local-customize-alist		. 'gretl-customize-alist)
     (inferior-ess-program		. "gretlcli")
-    (inferior-ess-font-lock-defaults	. gretl-font-lock-defaults)
     (ess-get-help-topics-function	. 'gretl-get-help-topics-function)
+    (font-lock-defaults		  . '(gretl-font-lock-keywords))
     (ess-load-command   		. "open \"%s\"\n")
     ;; (ess-dump-error-re			. "in \\w* at \\(.*\\):[0-9]+")
     ;; (ess-error-regexp			. "\\(^\\s-*at\\s-*\\(?3:.*\\):\\(?2:[0-9]+\\)\\)")
@@ -508,14 +505,13 @@ end keywords as associated values.")
     (ess-dialect			. "gretl")
     (ess-suffix				. "inp")
     (ess-dump-filename-template		. (replace-regexp-in-string
-					   "S$" ess-suffix ; in the one from custom:
-					   ess-dump-filename-template-proto))
+					                   "S$" ess-suffix ; in the one from custom:
+					                   ess-dump-filename-template-proto))
     (ess-mode-syntax-table		. gretl-syntax-table)
     (ess-mode-editing-alist	        . gretl-editing-alist)
     (ess-change-sp-regexp		. nil );ess-r-change-sp-regexp)
     (ess-help-sec-regex			. ess-help-r-sec-regex)
     (ess-help-sec-keys-alist		. ess-help-r-sec-keys-alist)
-    (ess-loop-timeout			. ess-S-loop-timeout);fixme: dialect spec.
     (ess-function-pattern		. ess-r-function-pattern)
     (ess-object-name-db-file		. "ess-r-namedb.el" )
     ;; (ess-imenu-mode-function		. nil)
@@ -530,7 +526,7 @@ end keywords as associated values.")
     )
   "Variables to customize for Gretl -- set up later than emacs initialization.")
 
-;; (defcustom inferior-gretl-program-name "gretlcli"
+;; (defcustom inferior-gretl-program "gretlcli"
 ;;   "*The program to use for running gretl scripts."
 ;;   :type 'string
 ;;   :group 'ess-gretl)
@@ -548,8 +544,6 @@ been created using the variable `ess-r-versions'."
   :group 'ess-gretl
   :type 'string)
 
-(defvar gretl-basic-offset 4)
-
 ;;;###autoload
 (defun gretl-mode  (&optional proc-name)
   "Major mode for editing gretl source.  See `ess-mode' for more help."
@@ -567,8 +561,8 @@ been created using the variable `ess-r-versions'."
   (set (make-local-variable 'end-of-defun-function) 'ess-end-of-function)
   ;; (local-set-key  "\t" 'gretl-indent-line) ;; temp workaround
   ;; (set (make-local-variable 'indent-line-function) 'gretl-indent-line)
- ;; (ess-imenu-gretl)
-  (run-hooks 'gretl-mode-hook))
+  ;; (ess-imenu-gretl)
+  (run-mode-hooks 'gretl-mode-hook))
 
 
 (defvar ess-gretl-post-run-hook nil
@@ -585,33 +579,33 @@ Optional prefix (C-u) allows to set command line arguments, such as
 If you have certain command line arguments that should always be passed
 to gretl, put them in the variable `inferior-gretl-args'."
   (interactive "P")
-  ;; get settings, notably inferior-ess-r-program-name :
-  ;; (if (null inferior-gretl-program-name)
-  ;;     (error "'inferior-gretl-program-name' does not point to 'gretl-release-basic' executable")
-    (setq ess-customize-alist gretl-customize-alist)
-    (ess-write-to-dribble-buffer   ;; for debugging only
-     (format
-      "\n(Gretl): ess-dialect=%s, buf=%s"
-      ess-dialect (current-buffer)))
-    (let* ((r-start-args
-	    (concat inferior-gretl-args " " ; add space just in case
-		    (if start-args
-			(read-string
-			 (concat "Starting Args [other than `"
-				 inferior-gretl-args
-				 "'] ? "))
-		      nil))))
-      (inferior-ess r-start-args)
-      (set (make-local-variable 'indent-line-function) 'gretl-indent-line)
-      (set (make-local-variable 'gretl-basic-offset) 4)
-      (setq indent-tabs-mode nil)
-      (goto-char (point-max))
-      ;; (if inferior-ess-language-start
-      ;; 	(ess-eval-linewise inferior-ess-language-start
-      ;; 			   nil nil nil 'wait-prompt)))
-      (with-ess-process-buffer nil
-        (run-mode-hooks 'ess-gretl-post-run-hook))
-      ))
+  ;; get settings, notably inferior-ess-r-program :
+  ;; (if (null inferior-gretl-program)
+  ;;     (error "'inferior-gretl-program' does not point to 'gretl-release-basic' executable")
+  (setq ess-customize-alist gretl-customize-alist)
+  (ess-write-to-dribble-buffer   ;; for debugging only
+   (format
+    "\n(Gretl): ess-dialect=%s, buf=%s"
+    ess-dialect (current-buffer)))
+  (let* ((r-start-args
+	  (concat inferior-gretl-args " " ; add space just in case
+		  (if start-args
+		      (read-string
+		       (concat "Starting Args [other than `"
+			       inferior-gretl-args
+			       "'] ? "))
+		    nil))))
+    (inferior-ess r-start-args)
+    (set (make-local-variable 'indent-line-function) 'gretl-indent-line)
+    (set (make-local-variable 'gretl-basic-offset) 4)
+    (setq indent-tabs-mode nil)
+    (goto-char (point-max))
+    ;; (if inferior-ess-language-start
+    ;; 	(ess-eval-linewise inferior-ess-language-start
+    ;; 			   nil nil nil 'wait-prompt)))
+    (with-ess-process-buffer nil
+      (run-mode-hooks 'ess-gretl-post-run-hook))
+    ))
 
 
 ;;;; IMENU
