@@ -2,8 +2,7 @@
 ;;
 ;; Copyright (C) 2012-2015 Vitalie Spinu and the ESS Core team.
 ;;
-;; Filename: ess-julia.el
-;; Author: Vitalie Spinu (based on julia-mode.el from julia-lang project)
+;; Author: Vitalie Spinu
 ;; Maintainer: Vitalie Spinu
 ;; Created: 02-04-2012 (ESS 12.03)
 ;; Keywords: ESS, julia
@@ -24,14 +23,14 @@
 ;; GNU General Public License for more details.
 ;;
 ;; A copy of the GNU General Public License is available at
-;; http://www.r-project.org/Licenses/
+;; https://www.r-project.org/Licenses/
 ;;
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;;; Commentary:
 ;;
-;;  Customise inferior-julia-program-name to point to your julia binary
+;;  Customise inferior-julia-program to point to your julia binary
 ;;  and start the inferior with M-x julia.
 ;;
 ;;  As of Sept 2015, this file depends heavily on julia-mode.el from the Julia
@@ -45,18 +44,19 @@
 (require 'compile)
 (require 'ess-utils)
 (require 'ess-r-mode)
+(require 'julia-mode)
 
-;;;--- ALL the following only if  julia-mode is found and loaded correctly : ----------
-(condition-case nil
-      (progn
-        (require 'julia-mode)
-        (when (featurep 'julia-mode)
+(defvar ac-prefix)
+(declare-function company-in-string-or-comment "company")
+(declare-function company-doc-buffer "company")
+
+(defcustom inferior-julia-args ""
+  "String of arguments (see 'julia --help') used when starting julia."
+  :group 'ess-julia
+  :type 'string)
 
 (eval-when-compile
-  (require 'cl))
-
-(autoload 'inferior-ess "ess-inf" "Run an ESS process.")
-(autoload 'ess-mode     "ess-mode" "Edit an ESS process.")
+  (require 'cl-lib))
 
 (defun ess-julia-send-string-function (process string visibly)
   "Send the Julia STRING to the PROCESS.
@@ -88,11 +88,11 @@ VISIBLY is not currently used."
 (defvar ess-julia--manual-topics nil)
 (defun ess-julia-manual-lookup-function (&rest args) ; args are not used
   (interactive)
-  "Look up topics at http://docs.julialang.org/en/latest/manual/"
+  "Look up topics at https://docs.julialang.org/en/latest/manual/"
   ;; <li class="toctree-l1"><a class="reference internal" href="introduction/">Introduction</a></li>
   (let* ((pages (or ess-julia--manual-topics
                     (setq ess-julia--manual-topics
-                          (ess-julia--retrive-topics "http://docs.julialang.org/en/latest/manual/"))))
+                          (ess-julia--retrive-topics "https://docs.julialang.org/en/latest/manual/"))))
          (page (ess-completing-read "Lookup:" pages nil t)))
     (browse-url (get-text-property 1 :manual page))))
 
@@ -201,8 +201,8 @@ objects from that MODULE."
         (push (cons (match-string 1) (match-string 2)) list))
       (when cache?
         (let ((objects (process-get proc 'objects)))
-         (push (cons obj list) objects)
-         (process-put proc 'objects objects)))
+          (push (cons obj list) objects)
+          (process-put proc 'objects objects)))
       list)))
 
 (defun ess-julia-get-object-help-string (sym)
@@ -227,8 +227,11 @@ objects from that MODULE."
   "Auto-completion source for julia objects")
 
 (defun ess-ac-julia-objects ()
+  (require 'auto-complete)
   (ess-julia-objects ac-prefix))
 
+(declare-function company-begin-backend "company.el")
+
 (defun company-ess-julia-objects (command &optional arg &rest ignored)
   (interactive (list 'interactive))
   (cl-case command
@@ -266,7 +269,7 @@ to look up any doc strings."
              (not (ess-process-get 'busy)))
     (let ((funname (or (and ess-eldoc-show-on-symbol ;; aggressive completion
                             (symbol-name (ess-symbol-at-point)))
-                       (car (ess--funname.start)))))
+                       (car (ess--fn-name-start)))))
       (when funname
         (let* ((args (copy-sequence (nth 2 (ess-function-arguments funname))))
                (W (- (window-width (minibuffer-window)) (+ 4 (length funname))))
@@ -303,12 +306,12 @@ to look up any doc strings."
     (inferior-ess-secondary-prompt . nil)
     (inferior-ess-prompt           . "\\w*> ")
     (ess-local-customize-alist     . 'ess-julia-customize-alist)
-    (inferior-ess-program          . inferior-julia-program-name)
+    (inferior-ess-program          . inferior-julia-program)
     (ess-get-help-topics-function  . 'ess-julia-get-help-topics)
-    (ess-help-web-search-command   . "http://docs.julialang.org/en/latest/search/?q=%s")
+    (ess-help-web-search-command   . "https://docs.julialang.org/en/latest/search/?q=%s")
     (ess-manual-lookup-command     . 'ess-julia-manual-lookup-function)
     ;; (ess-reference-lookup-command       . 'ess-julia-reference-lookup-function)
-    (ess-load-command              . "include(\"%s\")\n")
+    (ess-load-command              . "include(expanduser(\"%s\"))\n")
     (ess-funargs-command           . "ESS.fun_args(\"%s\")\n")
     (ess-dump-error-re             . "in \\w* at \\(.*\\):[0-9]+")
     (ess-error-regexp              . "\\(^\\s-*at\\s-*\\(?3:.*\\):\\(?2:[0-9]+\\)\\)")
@@ -332,7 +335,6 @@ to look up any doc strings."
     (ess-change-sp-regexp          . nil );ess-r-change-sp-regexp)
     (ess-help-sec-regex            . ess-help-r-sec-regex)
     (ess-help-sec-keys-alist       . ess-help-r-sec-keys-alist)
-    (ess-loop-timeout              . ess-S-loop-timeout);fixme: dialect spec.
     (ess-function-pattern          . ess-r-function-pattern)
     (ess-object-name-db-file       . "ess-jl-namedb.el" )
     (ess-smart-operators           . ess-r-smart-operators)
@@ -350,11 +352,6 @@ to look up any doc strings."
     )
   "Variables to customize for Julia -- set up later than emacs initialization.")
 
-(defcustom inferior-julia-args ""
-  "String of arguments (see 'julia --help') used when starting julia."
-  :group 'ess-julia
-  :type 'string)
-
 (defvar ess-julia-completion-syntax-table
   (let ((table (make-syntax-table ess-r-syntax-table)))
     (modify-syntax-entry ?. "_" table)
@@ -375,11 +372,9 @@ It makes underscores and dots word constituent chars.")
   (add-hook 'completion-at-point-functions 'ess-filename-completion nil 'local)
   (if (fboundp 'ess-add-toolbar) (ess-add-toolbar))
   (set (make-local-variable 'end-of-defun-function) 'ess-end-of-function)
-
-  (set (make-local-variable 'ess-julia-basic-offset) 4)
   (setq imenu-generic-expression ess-julia-imenu-generic-expression)
   (imenu-add-to-menubar "Imenu-jl")
-  (run-hooks 'ess-julia-mode-hook))
+  (run-mode-hooks 'ess-julia-mode-hook))
 
 (defvar ess-julia-mode-hook nil)
 (defvar ess-julia-post-run-hook nil
@@ -393,23 +388,23 @@ Optional prefix (C-u) allows to set command line arguments, such as
 If you have certain command line arguments that should always be passed
 to julia, put them in the variable `inferior-julia-args'."
   (interactive "P")
-  ;; get settings, notably inferior-julia-program-name :
-  (if (null inferior-julia-program-name)
-      (error "'inferior-julia-program-name' does not point to 'julia' or 'julia-basic' executable")
+  ;; get settings, notably inferior-julia-program :
+  (if (null inferior-julia-program)
+      (error "'inferior-julia-program' does not point to 'julia' or 'julia-basic' executable")
     (setq ess-customize-alist ess-julia-customize-alist)
     (ess-write-to-dribble-buffer   ;; for debugging only
      (format
       "\n(julia): ess-dialect=%s, buf=%s, start-arg=%s\n current-prefix-arg=%s\n"
       ess-dialect (current-buffer) start-args current-prefix-arg))
     (let* ((jl-start-args
-	    (concat inferior-julia-args " " ; add space just in case
-		    (if start-args
-			(read-string
+	        (concat inferior-julia-args " " ; add space just in case
+		            (if start-args
+			            (read-string
                          (concat "Starting Args"
                                  (if inferior-julia-args
                                      (concat " [other than '" inferior-julia-args "']"))
                                  " ? "))
-		      nil))))
+		              nil))))
       (inferior-ess jl-start-args)
 
       (remove-hook 'completion-at-point-functions 'ess-filename-completion 'local) ;; should be first
@@ -419,7 +414,6 @@ to julia, put them in the variable `inferior-julia-args'."
       (setq comint-input-sender 'ess-julia-input-sender)
 
       (ess--tb-start)
-      (set (make-local-variable 'ess-julia-basic-offset) 4)
       ;; remove ` from julia's logo
       (goto-char (point-min))
       (while (re-search-forward "`" nil t)
@@ -433,11 +427,9 @@ to julia, put them in the variable `inferior-julia-args'."
       ;; --> julia helpers from ../etc/ess-julia.jl :
       (ess--inject-code-from-file (format "%sess-julia.jl" ess-etc-directory))
       (with-ess-process-buffer nil
-        (run-mode-hooks 'ess-julia-post-run-hook))
-      )))
+        (run-mode-hooks 'ess-julia-post-run-hook)))))
 
 (add-to-list 'auto-mode-alist '("\\.jl\\'" . ess-julia-mode))
 
-))  (error nil))
 (provide 'ess-julia)
 ;;; ess-julia.el ends here
