@@ -21,7 +21,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; A copy of the GNU General Public License is available at
-;; http://www.r-project.org/Licenses/
+;; https://www.r-project.org/Licenses/
 ;;
 ;;; Commentary:
 ;;
@@ -30,8 +30,17 @@
 
 ;;; ElDoc
 
-(require 'cl)
+(require 'cl-lib)
 (require 'ess-utils)
+(require 'ess-inf)
+(require 'ess-help)
+
+(defvar ac-auto-start)
+(defvar ac-prefix)
+(defvar ac-point)
+(defvar ac-use-comphist)
+(declare-function company-begin-backend "company")
+(declare-function company-doc-buffer "company")
 
 (defun ess-r-eldoc-function ()
   "Return the doc string, or nil.
@@ -42,7 +51,7 @@ to look up any doc strings."
     (let* ((proc (ess-get-next-available-process))
            (funname (and proc (or (and ess-eldoc-show-on-symbol ;; Aggressive completion
                                        (thing-at-point 'symbol))
-                                  (car (ess--funname.start))))))
+                                  (car (ess--fn-name-start))))))
       (when funname
         (let* ((args (ess-function-arguments funname proc))
                (bargs (cadr args))
@@ -136,7 +145,7 @@ to look up any doc strings."
 (defun ess-r-object-completion ()
   "Return completions at point in a format required by `completion-at-point-functions'."
   (if (ess-make-buffer-current)
-      (let* ((funstart (cdr (ess--funname.start)))
+      (let* ((funstart (cdr (ess--fn-name-start)))
              (completions (ess-r-get-rcompletions funstart))
              (token (pop completions)))
         (when completions
@@ -160,8 +169,13 @@ or \\[ess-internal-complete-object-name] otherwise."
       (message "No ESS process associated with current buffer")
       nil)))
 
+(defun ess-list-object-completions nil
+  "List all possible completions of the object name at point."
+  (interactive)
+  (ess-complete-object-name))
+
 (defun ess-complete-object-name-deprecated ()
-  "Gives a deprecated message "
+  "Gives a deprecated message."
   (interactive)
   (ess-complete-object-name)
   (message "C-c TAB is deprecated, completions has been moved to [M-TAB] (aka C-M-i)")
@@ -174,7 +188,7 @@ The object is compared against those objects known by
 `ess-get-object-list' and any additional characters up to ambiguity are
 inserted.  Completion only works on globally-known objects (including
 elements of attached data frames), and thus is most suitable for
-interactive command-line entry, and not so much for function editing
+interactive `command-line' entry, and not so much for function editing
 since local objects (e.g. argument names) aren't known.
 
 Use \\[ess-resynch] to re-read the names of the attached directories.
@@ -186,14 +200,9 @@ command may be necessary if you modify an attached dataframe."
   (ess-make-buffer-current)
   (if (memq (char-syntax (preceding-char)) '(?w ?_))
       (let* ((comint-completion-addsuffix nil)
-             (end (point))
-             (buffer-syntax (syntax-table))
-             (beg (unwind-protect
-                      (save-excursion
-                        (set-syntax-table ess-mode-syntax-table)
-                        (backward-sexp 1)
-                        (point))
-                    (set-syntax-table buffer-syntax)))
+             (bounds (ess-bounds-of-symbol))
+             (beg (car bounds))
+             (end (cdr bounds))
              (full-prefix (buffer-substring beg end))
              (pattern full-prefix)
              ;; See if we're indexing a list with `$'
@@ -229,7 +238,7 @@ command may be necessary if you modify an attached dataframe."
                              ;;    options(error=recover) :
                              (ess-get-object-list ess-current-process-name)))))
         ;; always return a non-nil value to prevent history expansions
-        (or (comint-dynamic-simple-complete  pattern components) 'none))))
+        (or (completion-in-region beg end components) 'none))))
 
 (defun ess-r-get-rcompletions (&optional start end prefix allow-3-dots)
   "Call R internal completion utilities (rcomp) for possible completions.
@@ -251,7 +260,6 @@ token.  Needs version of R >= 2.7.0."
          (cmd (if allow-3-dots
                   (concat call1 "\n")
                 (concat "local({ r <- " call1 "; r[r != '...='] })\n"))))
-
     (ess-get-words-from-vector cmd)))
 
 (defun ess-r-complete-object-name ()
@@ -260,14 +268,11 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
   (interactive)
   (let ((possible-completions (ess-r-get-rcompletions))
         token-string)
-    ;; If there are no possible-completions, should return nil, so
-    ;; that when this function is called from
-    ;; comint-dynamic-complete-functions, other functions can also be
-    ;; tried.
     (when possible-completions
       (setq token-string (pop possible-completions))
-      (or (comint-dynamic-simple-complete token-string
-                                          possible-completions)
+      (or (completion-in-region (- (point) (length token-string))
+                                (point)
+                                possible-completions)
           'none))))
 
 (defvar ess--cached-sp-objects nil)
@@ -303,13 +308,12 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
 
 ;;; ARGUMENTS
 
+(define-obsolete-variable-alias 'ess-ac-R-argument-suffix 'ess-R-argument-suffix "15.3")
 (defcustom ess-R-argument-suffix " = "
   "Suffix appended by `ac-source-R' and `ac-source-R-args' to candidates."
   :group 'R
   :type 'string)
 
-(define-obsolete-variable-alias 'ess-ac-R-argument-suffix 'ess-R-argument-suffix "15.3")
-
 (defvar ess-r--funargs-pre-cache
   '(("plot"
      (("graphics")
@@ -329,7 +333,7 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
       ("x" "..." "digits" "signif.stars" "intercept" "tol" "se" "sort" "verbose" "indent" "style" ".bibstyle" "prefix" "vsep" "minlevel" "quote" "right" "row.names" "max" "na.print" "print.gap"
        "useSource" "diag" "upper" "justify" "title" "max.levels" "width" "steps" "showEnv" "newpage" "vp" "cutoff" "max.level" "give.attr" "units" "abbrCollate" "print.x" "deparse" "locale" "symbolic.cor"
        "loadings" "zero.print" "calendar"))))
-  "Alist of cached arguments for very time consuming functions.")
+  "Alist of cached arguments for time consuming functions.")
 
 
 ;;; HELP
@@ -356,7 +360,7 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
   (let ((proc (or proc (ess-get-next-available-process))))
     (if (null proc)
         "No free ESS process found"
-      (let ((fun (car ess--funname.start)))
+      (let ((fun (car ess--fn-name-start-cache)))
         (with-current-buffer (ess-command (format ".ess_arg_help('%s','%s')\n" sym fun)
                                           nil nil nil nil proc)
           (goto-char (point-min))
@@ -365,7 +369,7 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
 
 
 ;;; COMPANY
-;;; http://company-mode.github.io/
+;;; https://company-mode.github.io/
 
 (defun company-R-objects (command &optional arg &rest ignored)
   (interactive (list 'interactive))
@@ -395,7 +399,7 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
                       prefix))))))
     (candidates (let* ((proc (ess-get-next-available-process))
                        (args (delete "..." (nth 2 (ess-function-arguments
-                                                   (car ess--funname.start) proc))))
+                                                   (car ess--fn-name-start-cache) proc))))
                        (args (mapcar (lambda (a) (concat a ess-R-argument-suffix))
                                      args)))
                   (all-completions arg args)))
@@ -411,6 +415,34 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
     (require-match 'never)
     (doc-buffer (company-doc-buffer (ess-r-get-arg-help-string arg)))))
 
+(defvar-local ess-r--installed-packages-cache nil)
+
+(defun ess-r-installed-packages ()
+  "Return a list of currently installed R packages.
+The value is cached once per session and is not updated if new
+packages are installed."
+  (let ((proc (ess-get-next-available-process)))
+    (when proc
+      (with-current-buffer (process-buffer proc)
+        (or ess-r--installed-packages-cache
+            (setq ess-r--installed-packages-cache
+                  (ess-get-words-from-vector
+                   "print(unlist(lapply(.libPaths(), dir)), max=1e6)\n")))))))
+
+;; completion for library names -- only active within 'library(...)'
+(defun company-R-library (command &optional arg &rest ignored)
+  (interactive (list 'interactive))
+  (cl-case command
+    (interactive (company-begin-backend 'company-R-library))
+    (prefix (and (member (car (ess--fn-name-start 'symbol))
+                         '("library" "require"))
+                 (let ((start (ess-symbol-start)))
+                   (and start (buffer-substring start (point))))))
+    (candidates (all-completions arg (ess-r-installed-packages)))
+    (annotation "<pkg>")
+    (duplicates nil)
+    (sorted t)))
+
 
 ;;; AC SOURCES
 ;;; http://cx4a.org/software/auto-complete/index.html
@@ -421,7 +453,7 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
     (candidates . ess-ac-candidates)
     ;; (action  . ess-ac-action-args) ;; interfere with ac-fallback mechanism on RET (which is extremely annoing in inferior buffers)
     (document   . ess-ac-help))
-  "Combined ad-completion source for R function arguments and R objects")
+  "Combined ad-completion source for R function arguments and R objects.")
 
 (defun ess-ac-start ()
   (when (ess-process-live-p)
@@ -429,7 +461,7 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
         (ess-symbol-start))))
 
 (defun ess-ac-candidates ()
-  "OBJECTS + ARGS"
+  "OBJECTS + ARGS."
   (let ((args (ess-ac-args)))
     ;; sort of intrusive but right
     (if (and ac-auto-start
@@ -450,10 +482,10 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
     ;; (requires   . 2)
     (candidates . ess-ac-objects)
     (document   . ess-r-get-object-help-string))
-  "Auto-completion source for R objects")
+  "Auto-completion source for R objects.")
 
 (defun ess-ac-objects (&optional no-kill)
-  "Get all cached objects"
+  "Get all cached objects."
  (let ((aprf ac-prefix))
    (when (and aprf (ess-process-live-p))
      (unless no-kill ;; workaround
@@ -467,13 +499,13 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
     (candidates . ess-ac-args)
     ;; (action     . ess-ac-action-args)
     (document   . ess-r-get-arg-help-string))
-  "Auto-completion source for R function arguments")
+  "Auto-completion source for R function arguments.")
 
 (defun ess-ac-args ()
   "Get the args of the function when inside parentheses."
-  (when  (and ess--funname.start ;; set in a call to ess-arg-start
+  (when  (and ess--fn-name-start-cache ;; set in a call to ess-arg-start
               (ess-process-live-p))
-    (let ((args (nth 2 (ess-function-arguments (car ess--funname.start)))))
+    (let ((args (nth 2 (ess-function-arguments (car ess--fn-name-start-cache)))))
       (if args
           (set (make-local-variable 'ac-use-comphist) nil)
         (kill-local-variable 'ac-use-comphist))
@@ -485,3 +517,5 @@ To be used instead of ESS' completion engine for R versions >= 2.7.0."
   "getArgHelp('%s','%s')")
 
 (provide 'ess-r-completion)
+
+;;; ess-r-completion.el ends here
