@@ -19,7 +19,7 @@
 ;; GNU General Public License for more details.
 
 ;; A copy of the GNU General Public License is available at
-;; http://www.r-project.org/Licenses/
+;; https://www.r-project.org/Licenses/
 
 ;;; Commentary:
 
@@ -30,7 +30,8 @@
 (require 'ess-utils)
 (require 'regexp-opt)
 
-(with-no-warnings (require 'cl)) ; instead of cl-lib so we support Emacs 24.2
+(eval-when-compile
+  (require 'cl-lib))
 
 
 ;;*;; Utils
@@ -53,6 +54,9 @@
         (progn (up-list N) t))
     (error nil)))
 
+(defun ess-backward-up-list (&optional N)
+  (ess-up-list (- (or N 1))))
+
 (defun ess-forward-char (&optional N)
   (unless (= (point) (point-max))
     (forward-char (or N 1))
@@ -64,8 +68,8 @@
     t))
 
 (defun ess-goto-char (pos)
-  "Go to `pos' if it is non-nil.
-If `pos' is nil, return nil.  Otherwise return `pos' itself."
+  "Go to POS if it is non-nil.
+If POS is nil, return nil.  Otherwise return position itself."
   (when pos
     (goto-char pos)))
 
@@ -93,7 +97,7 @@ be advised"
                 (goto-char orig-point))))))
 
 (defmacro ess-while (test &rest body)
-  "Like (while) but return `t' when body gets executed once."
+  "Like `while' for TEST but return t when BODY gets executed once."
   (declare (indent 1)
            (debug (&rest form)))
   `(let (executed)
@@ -110,6 +114,7 @@ be advised"
      (ess-back-to-indentation)
      (progn ,@body)))
 
+(defvar containing-sexp)
 (defmacro ess-at-containing-sexp (&rest body)
   (declare (indent 0)
            (debug (&rest form)))
@@ -120,13 +125,12 @@ be advised"
      (progn ,@body)))
 
 (defmacro ess-any (&rest forms)
-  "Evaluates all arguments and return non-nil if one of the
-arguments is non-nil. This is useful to trigger
-side-effects. FORMS follows the same syntax as arguments to
-`(cond)'."
+  "Evaluate all arguments and return non-nil if one of the arguments is non-nil.
+This is useful to trigger side-effects. FORMS follows the same
+syntax as arguments to `cond'."
   (declare (indent 0) (debug nil))
   `(let ((forms (list ,@(mapcar (lambda (form) `(progn ,@form)) forms))))
-     (some 'identity (mapcar 'eval forms))))
+     (cl-some 'identity (mapcar 'eval forms))))
 
 (defun ess-char-syntax (string)
   (char-to-string (char-syntax (string-to-char string))))
@@ -143,7 +147,7 @@ side-effects. FORMS follows the same syntax as arguments to
   (ess-token-type (ess-refine-token token)))
 
 (defun ess-token-after (&optional token)
-  "Returns next token.
+  "Return next TOKEN.
 Cons cell containing the token type and string representation."
   (save-excursion
     (when token
@@ -151,7 +155,7 @@ Cons cell containing the token type and string representation."
     (ess-jump-token)))
 
 (defun ess-token-before (&optional token)
-  "Returns previous token.
+  "Return previous TOKEN.
 Cons cell containing the token type and string representation."
   (save-excursion
     (when token
@@ -185,6 +189,12 @@ Cons cell containing the token type and string representation."
       (list (ess-token--cons token-type token-value)
             (cons (point) token-end)))))
 
+(defsubst ess-climb-token--char (&rest chars)
+  (ess-while (and chars
+                  (eq (char-before) (car chars))
+                  (ess-backward-char))
+    (setq chars (cdr chars))))
+
 ;; Difficult to use regexps here because we want to match greedily
 ;; backward
 (defun ess-climb-token--operator ()
@@ -216,12 +226,6 @@ Cons cell containing the token type and string representation."
              (ess-climb-token--char ?: ?:))))
     'self))
 
-(defsubst ess-climb-token--char (&rest chars)
-  (ess-while (and chars
-                  (eq (char-before) (car chars))
-                  (ess-backward-char))
-    (setq chars (cdr chars))))
-
 (defun ess-climb-token--back-and-forth ()
   (let ((limit (point)))
     (when (ess-skip-token-backward)
@@ -253,9 +257,8 @@ Cons cell containing the token type and string representation."
 
 (defun ess-jump-token (&optional type string)
   "Consume a token forward.
-Returns a cons cell containing the token type and the token
-string content. Returns nil when the end of the buffer is
-reached."
+Return a cons cell containing the token type and the token string
+content. Return nil when the end of the buffer is reached."
   (ess-save-excursion-when-nil
     (ess-skip-blanks-forward t)
     (let* ((token-start (point))
@@ -356,7 +359,7 @@ reached."
       (ess-token-after= '("," ";"))
       (and (ess-token-after= "identifier")
            (not (memq (char-syntax (char-before)) '(?w ?_))))
-      (progn (/= (skip-syntax-backward ".") 0)
+      (progn (skip-syntax-backward ".")
              (ess-token-operator-p (ess-token-after)))
       (/= (skip-syntax-backward "w_") 0)))
 
@@ -727,7 +730,7 @@ reached."
 
 (defun backward-ess-r-sexp ()
   (interactive)
-  (error "todo"))
+  (error "Todo"))
 
 (defun ess-parser-tree-assoc (key tree)
   (let ((next tree)
@@ -748,8 +751,8 @@ reached."
 
 ;;*;; Point predicates
 
-(defun ess-within-call-p (&optional call)
-  "Is point in a function or indexing call?"
+(defun ess-inside-call-p (&optional call)
+  "Return non-nil if point is in a function or indexing call."
   (let ((containing-sexp (or (bound-and-true-p containing-sexp)
                              (ess-containing-sexp-position))))
     (save-excursion
@@ -760,9 +763,9 @@ reached."
              (ess-up-list))
            (or (ess-behind-call-opening-p "(")
                (looking-at "\\["))
-           (ess-within-call-name-p call)))))
+           (ess-inside-call-name-p call)))))
 
-(defun ess-within-continuation-p ()
+(defun ess-inside-continuation-p ()
   (unless (or (looking-at ",")
               (ess-behind-call-opening-p "[[(]"))
     (or (save-excursion
@@ -775,36 +778,27 @@ reached."
           (and (ess-behind-operator-p)
                (not (ess-ahead-param-assign-p)))))))
 
-(defun ess-within-call-name-p (&optional call)
+(defun ess-inside-call-name-p (&optional call)
   (save-excursion
     (ess-climb-call-name call)))
 
-(defun ess-within-prefixed-block-p (&optional call)
-  "Is point in a prefixed block? Prefixed blocks refer to the
-blocks following function declarations, control flow statements,
-etc.
+(defun ess-inside-prefixed-block-p (&optional call)
+  "Return non-nil if point is in a prefixed block.
+Prefixed blocks refer to the blocks following function
+declarations, control flow statements, etc.
 
-If CALL not nil, check if the prefix corresponds to CALL. If nil,
-return the prefix."
+If CALL is not nil, check if the prefix corresponds to CALL. If
+nil, return the prefix."
   (save-excursion
     (ess-escape-prefixed-block call)))
 
-(defun ess-within-comment-p (&optional state)
-  (let ((state (or state (syntax-ppss))))
-        (eq (syntax-ppss-context state) 'comment)))
-
-(defun ess-within-string-p (&optional state)
-  (let ((state (or state (syntax-ppss))))
-        (eq (syntax-ppss-context state) 'string)))
-
 
 ;;*;; Syntactic Travellers and Predicates
 
 ;;;*;;; Blanks, Characters, Comments and Delimiters
 
 (defun ess-skip-blanks-backward (&optional newlines)
-  "Skip blanks and newlines backward, taking end-of-line comments
-into account."
+  "Skip blanks and newlines backward, taking end-of-line comments into account."
   (ess-any ((ess-skip-blanks-backward-1))
            ((when newlines
               (ess-while (and (/= (point) (point-min))
@@ -818,8 +812,7 @@ into account."
        (/= 0 (skip-syntax-backward " "))))
 
 (defun ess-skip-blanks-forward (&optional newlines)
-  "Skip blanks and newlines forward, taking end-of-line comments
-into account."
+  "Skip blanks and newlines forward, taking end-of-line comments into account."
   (ess-any ((/= 0 (skip-syntax-forward " ")))
            ((ess-while (and newlines
                             (= (point) (ess-code-end-position))
@@ -839,7 +832,7 @@ into account."
       (goto-char (match-end 0)))))
 
 (defun ess-escape-comment ()
-  (when (ess-within-comment-p)
+  (when (ess-inside-comment-p)
     (prog1 (comment-beginning)
      (skip-chars-backward "#+[ \t]*"))))
 
@@ -925,7 +918,7 @@ into account."
 (defun ess-behind-prefixed-block-p (&optional call)
   (if call
       (looking-at (concat call "[ \t]*("))
-    (some 'looking-at ess-prefixed-block-patterns)))
+    (cl-some 'looking-at ess-prefixed-block-patterns)))
 
 (defun ess-unbraced-block-p (&optional ignore-ifelse)
   "This indicates whether point is in front of an unbraced
@@ -937,16 +930,16 @@ position of the control flow function (if, for, while, etc)."
                   (not ignore-ifelse))
              (and (looking-at "(")
                   (ess-backward-sexp)
-                  (some 'looking-at ess-prefixed-block-patterns)
+                  (cl-some 'looking-at ess-prefixed-block-patterns)
                   (if ignore-ifelse
                       (not (looking-at "if\\b"))
                     t)))
          (point))))
 
 (defun ess-climb-block-prefix (&optional call ignore-ifelse)
-  "Climb the prefix of a prefixed block. Prefixed blocks refer to
-the blocks following function declarations, control flow
-statements, etc.
+  "Climb the prefix of a prefixed block.
+Prefixed blocks refer to the blocks following function
+declarations, control flow statements, etc.
 
 Should be called either in front of a naked block or in front
 of the curly brackets of a braced block.
@@ -1034,8 +1027,8 @@ return the prefix."
     (ess-climb-object)))
 
 (defun ess-ahead-param-assign-p ()
-  "Are we looking at a function argument? To be called just
-before the `=' sign."
+  "Return non-nil if looking at a function argument.
+To be called just before the `=' sign."
   (ess-refined-token= (ess-token-before) "param-assign"))
 
 (defun ess-behind-arg-p ()
@@ -1098,11 +1091,11 @@ before the `=' sign."
 
 (defun ess-step-to-first-arg ()
   (let ((containing-sexp (ess-containing-sexp-position)))
-    (cond ((ess-within-call-p)
+    (cond ((ess-inside-call-p)
            (goto-char containing-sexp)
            (forward-char)
            t)
-          ((ess-within-call-name-p)
+          ((ess-inside-call-name-p)
            (ess-jump-name)
            (ess-skip-blanks-forward)
            (forward-char)
@@ -1146,7 +1139,7 @@ before the `=' sign."
 
 (defun ess-escape-call (&optional call)
   (let ((containing-sexp (ess-containing-sexp-position)))
-    (if (ess-within-call-p)
+    (if (ess-inside-call-p)
         (ess-save-excursion-when-nil
           (goto-char containing-sexp)
           (ess-climb-chained-delims)
@@ -1177,7 +1170,7 @@ before the `=' sign."
 
 (defun ess-args-bounds (&optional marker)
   (let ((containing-sexp (ess-containing-sexp-position)))
-    (when (ess-within-call-p)
+    (when (ess-inside-call-p)
       (save-excursion
         (let ((beg (1+ containing-sexp))
               (call-beg (ess-at-containing-sexp
@@ -1327,7 +1320,7 @@ expression."
       ;; In calls, operators can start on newlines
       (let ((start-line (line-number-at-pos)))
         (when (ess-save-excursion-when-nil
-                (and (ess-within-call-p)
+                (and (ess-inside-call-p)
                      (ess-skip-blanks-forward t)
                      (/= (line-number-at-pos) start-line)
                      (ess-behind-operator-p)))
@@ -1492,10 +1485,10 @@ without curly braces."
   (save-excursion
     (and (ess-behind-call-p)
          (ess-jump-inside-call)
-         (some (lambda (arg)
-                 (string-match "^function\\b"
-                               (cdr arg)))
-               (ess-args-alist)))))
+         (cl-some (lambda (arg)
+                    (string-match "^function\\b"
+                                  (cdr arg)))
+                  (ess-args-alist)))))
 
 
 ;;;*;;; Names / Objects / Expressions
@@ -1511,9 +1504,9 @@ without curly braces."
       (if (and (memq (char-before) '(?` ?\" ?\'))
                (ess-backward-sexp))
           (setq climbed t)
-        (while (some (apply-partially '/= 0)
-                     `(,(skip-syntax-backward "w_")
-                       ,(skip-chars-backward "\"'")))
+        (while (cl-some (apply-partially '/= 0)
+                        `(,(skip-syntax-backward "w_")
+                          ,(skip-chars-backward "\"'")))
           (setq climbed t)))
       ;; Recurse if we find an indexing char
       (when (memq (char-before) '(?$ ?@))
@@ -1574,21 +1567,4 @@ without curly braces."
 
 (provide 'ess-r-syntax)
 
- ; Local variables section
-
-;;; This file is automatically placed in Outline minor mode.
-;;; The file is structured as follows:
-;;; Chapters:     ^L ;
-;;; Sections:    ;;*;;
-;;; Subsections: ;;;*;;;
-;;; Components:  defuns, defvars, defconsts
-;;;              Random code beginning with a ;;;;* comment
-
-;;; Local variables:
-;;; mode: emacs-lisp
-;;; outline-minor-mode: nil
-;;; mode: outline-minor
-;;; outline-regexp: "\^L\\|\\`;\\|;;\\*\\|;;;\\*\\|(def[cvu]\\|(setq\\|;;;;\\*"
-;;; End:
-
 ;;; ess-r-syntax.el ends here
