@@ -21,13 +21,13 @@
 ;;
 ;; You should have received a copy of the GNU General Public License
 ;; along with this program.  If not, see
-;; <http://www.gnu.org/licenses/>.
+;; <https://www.gnu.org/licenses/>.
 
 
 ;;; Commentary:
 
 ;; Lots of inspiration from doc-mode,
-;; http://nschum.de/src/emacs/doc-mode/
+;; https://nschum.de/src/emacs/doc-mode/
 ;;
 ;; Features::
 ;;
@@ -59,8 +59,7 @@
 
 ;; this *is* enabled now via ess-mode-hook in ./ess-site.el
 
-
-;;*;; Dependencies:
+;;; Code:
 
 (require 'ess-utils)
 (require 'ess-custom)
@@ -68,11 +67,11 @@
 (require 'hideshow)
 (require 'outline)
 (eval-when-compile
-  ;; We can't use cl-lib whilst supporting Emacs <= 24.2 users
-  (with-no-warnings (require 'cl)))
-(autoload 'Rd-preview-help "ess-rd" "[autoload]" t)
+  (require 'cl-lib))
 (require 'essddr "ess-rd.el")
 
+(defvar roxy-str)
+
 
 ;;*;; Roxy Minor Mode
 
@@ -96,11 +95,11 @@
               (regexp-opt ess-roxy-tags-param t)
               "\\)\\>")
      (1 'font-lock-keyword-face prepend))
-    (,(concat ess-roxy-re " *\\([@\\]"
+    (,(concat ess-roxy-re " *\\(@"
               (regexp-opt '("param" "importFrom" "importClassesFrom"
                             "importMethodsFrom")
-                          t)
-              "\\)\\>\\(?:[ \t]+\\(\\sw+\\)\\)?")
+                          'words)
+              "\\)\\(?:[ \t]+\\(\\(?:\\sw+,?\\)+\\)\\)")
      (1 'font-lock-keyword-face prepend)
      (3 'font-lock-variable-name-face prepend))
     (,(concat "[@\\]" (regexp-opt ess-roxy-tags-noparam t) "\\>")
@@ -108,95 +107,9 @@
     (,(concat ess-roxy-re)
      (0 'bold prepend))))
 
-(defvar ess-roxy-fontify-examples nil
-  "When non-nil, the `@examples' field is fontified as ordinary code.
-Experimental feature with known bugs.")
-
-(defun ess-roxy-extend-region-to-field (start end)
-  (if (or (progn
-            (goto-char start)
-            (ess-roxy-entry-p "examples"))
-          (progn
-            (goto-char end)
-            (ess-roxy-entry-p "examples")))
-      (let ((new-start (min start (ess-roxy-beg-of-field)))
-            (new-end (max end (ess-roxy-end-of-field))))
-        (cons new-start new-end))
-    (cons start end)))
-
-(defun ess-roxy-modify-examples (overlay after start end &optional length)
-  (when (and overlay after)
-    (ess-roxy-delete-examples-field overlay (min (1+ (overlay-end overlay))
-                                                 (point-max)))))
-
-(defun ess-roxy-insert-behind-examples (overlay after start end &optional length)
-  (when (and overlay after)
-    (ess-roxy-delete-examples-field overlay (1+ end))))
-
-(defun ess-roxy-delete-examples-field (overlay header-end)
-  (let ((inhibit-modification-hooks t))
-    (delete-overlay overlay)
-    (when (get-text-property header-end 'ess-roxy-examples)
-      (let ((field-end (next-single-property-change header-end 'ess-roxy-examples)))
-        (remove-list-of-text-properties header-end field-end
-                                        (list 'ess-roxy-examples
-                                              'ess-face-adjusted
-                                              'ess-adjust-face-background))))))
-
-(defun ess-roxy-syntax-propertize (start end)
-  (funcall
-   (syntax-propertize-rules
-    ;; Cache `@examples' field boundaries in text properties. Signal
-    ;; buffer and chunks for face adjustment.
-    ("^\\(#+'\\) +\\(@examples\\)[ \t\n]"
-     (1 (progn
-          (setq-local ess-buffer-has-chunks t)
-          (let ((field-start (1+ (match-end 2)))
-                (field-end (1+ (save-match-data
-                                 (ess-roxy-end-of-field)))))
-            (add-text-properties field-start field-end
-                                 (list 'ess-adjust-face-background t
-                                       'ess-roxy-examples t))
-            (unless (ess-find-overlay (match-beginning 2) 'ess-roxy-examples-header)
-              (let ((overlay (make-overlay (match-beginning 2) (match-end 2))))
-                (overlay-put overlay 'modification-hooks (list #'ess-roxy-modify-examples))
-                (overlay-put overlay 'insert-in-front-hooks (list #'ess-roxy-modify-examples))
-                (overlay-put overlay 'insert-behind-hooks (list #'ess-roxy-insert-behind-examples))
-                (overlay-put overlay 'ess-roxy-examples-header t))))
-          nil)))
-    ("^#+'"
-     ;; Remove comment and string properties of roxy prefix in fields
-     ;; that should be fontified as usual. Add `roxy-prefix' property
-     ;; so we can manually fontify the prefix as comment later on.
-     (0 (when (get-text-property (match-beginning 0) 'ess-roxy-examples)
-          (add-text-properties (match-beginning 0) (match-end 0)
-                               (list 'ess-roxy-prefix t
-                                     'font-lock-face 'font-lock-comment-face))
-          (string-to-syntax "-")))))
-   start end))
-
-(defun ess-roxy-fontify-region (start end loudly)
-  (prog1 (font-lock-default-fontify-region start end loudly)
-    (when (and ess-adjust-chunk-faces ess-buffer-has-chunks)
-      (let* ((prop 'ess-adjust-face-background)
-             (end (line-end-position))
-             (adjust-start (or (and (get-text-property start prop)
-                                    (previous-single-property-change start prop))
-                               (next-single-property-change start prop nil end)))
-             next-pos)
-        (while (progn
-                 (when (get-text-property adjust-start 'ess-face-adjusted)
-                   (setq adjust-start (next-single-property-change
-                                       adjust-start 'ess-face-adjusted nil end)))
-                 (< adjust-start end))
-          (setq next-pos (next-single-property-change adjust-start prop nil end))
-          (when (text-property-not-all adjust-start end 'ess-face-adjusted t)
-            (ess-adjust-face-background adjust-start next-pos))
-          (setq adjust-start (next-single-property-change next-pos prop nil end)))))))
-
-(defun ess-roxy-unfontify-region (start end)
-  (font-lock-default-unfontify-region start end)
-  (remove-list-of-text-properties start end (list 'ess-face-adjusted)))
+(defvar ess-roxy-fold-examples nil
+  "Whether to fold `@examples' when opening a buffer.
+Use you regular key for `outline-show-entry' to reveal it.")
 
 (define-minor-mode ess-roxy-mode
   "Minor mode for editing ROxygen documentation."
@@ -204,9 +117,7 @@ Experimental feature with known bugs.")
   (if ess-roxy-mode
       (progn
         (font-lock-add-keywords nil ess-roxy-font-lock-keywords)
-        (if (>= emacs-major-version 24)
-            (add-to-list 'completion-at-point-functions 'ess-roxy-tag-completion)
-          (add-to-list 'comint-dynamic-complete-functions 'ess-roxy-complete-tag))
+        (add-hook 'completion-at-point-functions #'ess-roxy-complete-tag 'local) ;; should be first
         ;; Hideshow Integration
         (when (and ess-roxy-hide-show-p (featurep 'hideshow))
           (hs-minor-mode 1)
@@ -214,15 +125,7 @@ Experimental feature with known bugs.")
             (ess-roxy-hide-all)))
         ;;  Outline Integration
         (when ess-roxy-fold-examples
-          (ess-roxy-hide-all-examples))
-        ;; Fontification
-        (when ess-roxy-fontify-examples
-          (add-hook 'syntax-propertize-extend-region-functions
-                    #'ess-roxy-extend-region-to-field
-                    'append 'local)
-          (setq-local syntax-propertize-function #'ess-roxy-syntax-propertize)
-          (setq-local font-lock-fontify-region-function #'ess-roxy-fontify-region)
-          (setq-local font-lock-unfontify-region-function #'ess-roxy-unfontify-region)))
+          (ess-roxy-hide-all-examples)))
     (when (and ess-roxy-hide-show-p
                (bound-and-true-p hs-minor-mode))
       (hs-show-all)
@@ -230,12 +133,13 @@ Experimental feature with known bugs.")
     (font-lock-remove-keywords nil ess-roxy-font-lock-keywords)
     (setq-local syntax-propertize-function nil)
     (setq-local font-lock-fontify-region-function nil)
-    (setq-local font-lock-unfontify-region-function nil)
-    (remove-hook 'syntax-propertize-extend-region-functions
-                 #'ess-roxy-extend-region-to-field
-                 'local))
+    (setq-local font-lock-unfontify-region-function nil))
   (when font-lock-mode
-    (font-lock-fontify-buffer))
+    (if (fboundp 'font-lock-flush)
+        (font-lock-flush)
+      ;; font-lock-fontify-buffer call can be removed when we drop
+      ;; support for Emacs older than 25.1
+      (with-no-warnings (font-lock-fontify-buffer))))
   ;; Autofill
   (setq-local paragraph-start (concat "\\(" ess-roxy-re "\\)*" paragraph-start))
   (setq-local paragraph-separate (concat "\\(" ess-roxy-re "\\)*" paragraph-separate))
@@ -245,10 +149,6 @@ Experimental feature with known bugs.")
 
 ;;*;; Outline Integration
 
-(defvar ess-roxy-fold-examples nil
-  "Whether to fold `@examples' when opening a buffer.
-Use you regular key for `outline-show-entry' to reveal it.")
-
 (defvar ess-roxy-outline-regexp "^#+' +@examples\\|^[^#]")
 
 (defun ess-roxy-substitute-outline-regexp (command)
@@ -257,6 +157,7 @@ Use you regular key for `outline-show-entry' to reveal it.")
                           outline-regexp)))
     (funcall command)))
 
+(declare-function outline-cycle "outline-magic")
 (defun ess-roxy-cycle-example ()
   (interactive)
   (unless (featurep 'outline-magic)
@@ -299,14 +200,14 @@ Use you regular key for `outline-show-entry' to reveal it.")
 ;;*;; Function definitions
 
 (defun ess-back-to-roxy ()
-  "Go to roxy prefix"
+  "Go to roxy prefix."
   (progn
     (end-of-line)
     (re-search-backward (concat ess-roxy-re " ?") (point-at-bol))
     (goto-char (match-end 0))))
 
 (defun ess-roxy-beg-of-entry ()
-  "Get point number at start of current entry, 0 if not in entry"
+  "Get point number at start of current entry, 0 if not in entry."
   (save-excursion
     (let (beg)
       (beginning-of-line)
@@ -319,7 +220,7 @@ Use you regular key for `outline-show-entry' to reveal it.")
       beg)))
 
 (defun ess-roxy-in-header-p ()
-  "true if point is the description / details field"
+  "True if point is the description / details field."
   (save-excursion
     (let ((res t)
           (cont (ess-roxy-entry-p)))
@@ -332,7 +233,7 @@ Use you regular key for `outline-show-entry' to reveal it.")
         )res)))
 
 (defun ess-roxy-beg-of-field ()
-  "Get point number at beginning of current field, 0 if not in entry"
+  "Get point number at beginning of current field, 0 if not in entry."
   (save-excursion
     (let (cont beg)
       (beginning-of-line)
@@ -352,7 +253,7 @@ Use you regular key for `outline-show-entry' to reveal it.")
       beg)))
 
 (defun ess-roxy-end-of-entry ()
-  " get point number at end of current entry, 0 if not in entry"
+  "Get point number at end of current entry, 0 if not in entry."
   (save-excursion
     (let ((end))
       (end-of-line)
@@ -366,7 +267,7 @@ Use you regular key for `outline-show-entry' to reveal it.")
       end)))
 
 (defun ess-roxy-end-of-field ()
-  "get point number at end of current field, 0 if not in entry"
+  "Get point number at end of current field, 0 if not in entry."
   (save-excursion
     (let ((end nil)
           (cont nil))
@@ -391,7 +292,7 @@ Use you regular key for `outline-show-entry' to reveal it.")
       end)))
 
 (defun ess-roxy-entry-p (&optional field)
-  "True if point is in a roxy entry"
+  "True if point is in a roxy entry."
   (and (save-excursion
          (beginning-of-line)
          (looking-at ess-roxy-re))
@@ -399,7 +300,7 @@ Use you regular key for `outline-show-entry' to reveal it.")
            (string= (ess-roxy-current-field) field))))
 
 (defun ess-roxy-narrow-to-field ()
-  "Go to to the start of current field"
+  "Go to to the start of current field."
   (interactive)
   (let ((beg (ess-roxy-beg-of-field))
         (end (ess-roxy-end-of-field)))
@@ -451,8 +352,9 @@ Use you regular key for `outline-show-entry' to reveal it.")
     (insert (make-string ess-indent-offset ? ))))
 
 (defun ess-roxy-goto-func-def ()
-  "put point at start of function either that the point is in or
-below the current roxygen entry, error otherwise"
+  "Put point at start of function.
+Go to the beginning of the current one or below the current
+roxygen entry, error otherwise"
   (if (ess-roxy-entry-p)
       (progn
         (ess-roxy-goto-end-of-entry)
@@ -461,16 +363,16 @@ below the current roxygen entry, error otherwise"
     (goto-char (car (ess-end-of-function)))))
 
 (defun ess-roxy-get-args-list-from-def ()
-  "get args list for current function"
+  "Get args list for current function."
   (save-excursion
     (ess-roxy-goto-func-def)
     (let ((args (ess-roxy-get-function-args)))
       (mapcar (lambda (x) (cons x '(""))) args))))
 
 (defun ess-roxy-insert-args (args &optional here)
-  "Insert an args list to the end of the roxygen entry for the
-function at point. if here is supplied start inputting
-`here'. Finish at end of line."
+  "Insert an ARGS list to the end of the current roxygen entry.
+If HERE is supplied start inputting `here'. Finish at end of
+line."
   (let* ((arg-des nil)
          (roxy-str (ess-roxy-guess-str)))
     (if (or (not here) (< here 1))
@@ -494,8 +396,8 @@ function at point. if here is supplied start inputting
               (fill-paragraph)))))))
 
 (defun ess-roxy-merge-args (fun ent)
-  "Take two args lists (alists) and return their union. Result
-holds all keys from both fun and ent but no duplicates and
+  "Take two args lists (alists) and return their union.
+The result holds all keys from both FUN and ENT but no duplicates and
 association from ent are preferred over entries from fun. Also,
 drop entries from ent that are not in fun and are associated with
 the empty string."
@@ -514,12 +416,11 @@ the empty string."
     (nreverse res-arg)))
 
 (defun ess-roxy-update-entry ()
-  "Update the entry at the point or the entry above the function
-which the point is in. Add a template empty roxygen documentation
-if no roxygen entry is available. The template can be customized
-via the variable `ess-roxy-template-alist'. The parameter
-descriptions can are filled if `ess-roxy-fill-param-p' is
-non-nil."
+  "Update the entry at point or the entry above the current function.
+Add a template empty roxygen documentation if no roxygen entry is
+available. The template can be customized via the variable
+`ess-roxy-template-alist'. The parameter descriptions can are
+filled if `ess-roxy-fill-param-p' is non-nil."
   (interactive)
   (save-excursion
     (let* ((args-fun (ess-roxy-get-args-list-from-def))
@@ -557,10 +458,9 @@ non-nil."
           (setq line-break "\n"))))))
 
 (defun ess-roxy-goto-end-of-entry ()
-  "Put point at the top of the entry at point or above the
-function at point. Return t if the point is left in a roxygen
-entry, otherwise nil. Error if point is not in function or
-roxygen entry."
+  "Put point at the top of the current entry or above the function at point.
+Return t if the point is left in a roxygen entry, otherwise nil.
+Error if point is not in function or roxygen entry."
   (if (not (ess-roxy-entry-p))
       (progn
         (goto-char (nth 0 (ess-end-of-function)))
@@ -572,8 +472,8 @@ roxygen entry."
     (forward-line) nil))
 
 (defun ess-roxy-goto-beg-of-entry ()
-  "put point at the top of the entry at point or above the
-function at point. Return t if the point is left in a roxygen
+  "Put point at the top of the entry at point or above the function at point.
+Return t if the point is left in a roxygen
 entry, otherwise nil. Error if point is not in function or
 roxygen entry."
   (if (not (ess-roxy-entry-p))
@@ -587,9 +487,9 @@ roxygen entry."
     (forward-line) nil))
 
 (defun ess-roxy-delete-args ()
-  "remove all args from the entry at point or above the function
-at point. Return 0 if no deletions were made other wise the point
-at where the last deletion ended"
+  "Remove all args from the entry at point or above the function at point.
+Return 0 if no deletions were made other wise the point at where
+the last deletion ended"
   (save-excursion
     (let* ((args nil)
            (cont t)
@@ -613,8 +513,7 @@ at where the last deletion ended"
       field-beg)))
 
 (defun ess-roxy-get-args-list-from-entry ()
-  "fill an args list from the entry above the function where the
-point is"
+  "Fill an args list from the entry above the function where the point is."
   (save-excursion
     (let* (args entry-beg field-beg field-end args-text arg-name desc)
       (if (ess-roxy-goto-end-of-entry)
@@ -654,7 +553,7 @@ region, otherwise prefix all lines with the roxy
 string. Convenient for editing example fields."
   (interactive "r")
   (unless (use-region-p)
-    (error "region is not active"))
+    (error "Region is not active"))
   (ess-roxy-roxy-region beg end (ess-roxy-entry-p)))
 
 (defun ess-roxy-roxy-region (beg end &optional on)
@@ -677,26 +576,25 @@ string. Convenient for editing example fields."
 `ess-roxy-package' to generate the Rd code for entry at point, place it
 in a temporary buffer and return that buffer."
   (let* ((beg (ess-roxy-beg-of-entry))
-        (tmpf (make-temp-file "ess-roxy"))
-        (roxy-buf (get-buffer-create " *RoxygenPreview*"))
-        (R-old-roxy
-         (concat
-          "..results <- roxygen2:::roc_process(rd_roclet(), parse.files(P), \"\");"
-          "cat(vapply(..results, function(x) roxygen2:::rd_out_cache$compute(x, format(x)), character(1)))" ))
-        (R-new-roxy
-         (concat
-          "..results <- roc_proc_text(rd_roclet(), readChar(P, file.info(P)$size));"
-          "cat(vapply(..results, format, character(1)))" ))
-        (out-rd-roclet
-         (cond ((string= "roxygen" ess-roxy-package)
-                "make.Rd2.roclet()$parse")
-               ;; must not line break strings to avoid getting +s in the output
-               ((string= "roxygen2" ess-roxy-package)
-                (concat "(function(P) { if(packageVersion('roxygen2') < '3.0.0') {"
-                        R-old-roxy "} else {" R-new-roxy "} })"))
-               (t (error "need to hard code the roclet output call for roxygen package '%s'"
-                         ess-roxy-package))))
-        )
+         (tmpf (make-temp-file "ess-roxy"))
+         (roxy-buf (get-buffer-create " *RoxygenPreview*"))
+         (R-old-roxy
+          (concat
+           "..results <- roxygen2:::roc_process(rd_roclet(), parse.files(P), \"\");"
+           "cat(vapply(..results, function(x) roxygen2:::rd_out_cache$compute(x, format(x)), character(1)))" ))
+         (R-new-roxy
+          (concat
+           "..results <- roc_proc_text(rd_roclet(), readChar(P, file.info(P)$size));"
+           "cat(vapply(..results, format, character(1)), \"\n\")" ))
+         (out-rd-roclet
+          (cond ((string= "roxygen" ess-roxy-package)
+                 "make.Rd2.roclet()$parse")
+                ;; must not line break strings to avoid getting +s in the output
+                ((string= "roxygen2" ess-roxy-package)
+                 (concat "(function(P) { if(packageVersion('roxygen2') < '3.0.0') {"
+                         R-old-roxy "} else {" R-new-roxy "} })"))
+                (t (error "Need to hard code the roclet output call for roxygen package '%s'"
+                          ess-roxy-package)))))
     (if (= beg 0)
         (error "Point is not in a Roxygen entry"))
     (save-excursion
@@ -708,15 +606,18 @@ in a temporary buffer and return that buffer."
                     (not (looking-at ess-roxy-re))))
         (append-to-file beg (point) tmpf))
       (ess-force-buffer-current)
-      (ess-command (concat "print(suppressWarnings(require(" ess-roxy-package
-                           ", quietly=TRUE)))\n")
-                   roxy-buf)
+      (unless (ess-boolean-command (concat "print(suppressWarnings(require(" ess-roxy-package
+                                           ", quietly=TRUE)))\n"))
+        (error (concat "Failed to load the " ess-roxy-package " package; "
+                       "in R, try  install.packages(\"" ess-roxy-package "\")")))
+      (ess-command (concat out-rd-roclet "(\"" tmpf "\")\n") roxy-buf)
       (with-current-buffer roxy-buf
-        (goto-char 1)
-        (if (search-forward-regexp "FALSE" nil t)
-            (error (concat "Failed to load the " ess-roxy-package " package; "
-                           "in R, try  install.packages(\"" ess-roxy-package "\")"))))
-      (ess-command (concat out-rd-roclet "(\"" tmpf "\")\n") roxy-buf))
+        ;; Kill characters up to % in case we missed stripping prompts
+        ;; or +'s:
+        (goto-char (point-min))
+        (when (re-search-forward "%" (line-end-position) t)
+          (backward-char)
+          (delete-region (line-beginning-position) (point)))))
     (delete-file tmpf)
     roxy-buf))
 
@@ -752,9 +653,10 @@ generate the text help page of the roxygen entry at point."
     (Rd-preview-help)))
 
 (defun ess-roxy-preview-Rd (&optional name-file)
-  "Use the connected R session and the roxygen package to
+  "Preview Rd for the roxygen entry at point.
+Use the connected R session and the roxygen package to
 generate the Rd code for the roxygen entry at point. If called
-with a non-nil `name-file' (e.g. universal argument C-u),
+with a non-nil NAME-FILE (\\[universal-argument]),
 also set the visited file name of the created buffer to
 facilitate saving that file."
   (interactive "P")
@@ -771,13 +673,13 @@ facilitate saving that file."
     (if (fboundp 'font-lock-ensure)
         (font-lock-ensure)
       ;; emacs <= 24.x.y:
-      (font-lock-fontify-buffer))))
+      (with-no-warnings (font-lock-fontify-buffer)))))
 
 
 (defun ess-roxy-guess-str (&optional not-here)
-  "guess the prefix used in the current roxygen block. If
-`not-here' is non-nil, guess the prefix for nearest roxygen
-block before the point"
+  "Guess the prefix used in the current roxygen block.
+If NOT-HERE is non-nil, guess the prefix for nearest roxygen
+block before the point."
   (save-excursion
     (if (ess-roxy-entry-p)
         (progn
@@ -790,7 +692,7 @@ block before the point"
       ess-roxy-str)))
 
 (defun ess-roxy-hide-block ()
-  "hide current roxygen comment block"
+  "Hide current roxygen comment block."
   (interactive)
   (save-excursion
     (let ((end-of-entry (ess-roxy-end-of-entry))
@@ -807,12 +709,12 @@ See `hs-show-block' and `ess-roxy-hide-block'."
      (ess-roxy-hide-block))))
 
 (defun ess-roxy-show-all ()
-  "Hide all Roxygen entries in current buffer. "
+  "Hide all Roxygen entries in current buffer."
   (interactive)
   (ess-roxy-hide-all t))
 
 (defun ess-roxy-hide-all (&optional show)
-  "Hide all Roxygen entries in current buffer. "
+  "Hide all Roxygen entries in current buffer."
   (interactive)
   (hs-life-goes-on
    (save-excursion
@@ -826,7 +728,7 @@ See `hs-show-block' and `ess-roxy-hide-block'."
          (forward-line 1))))))
 
 (defun ess-roxy-previous-entry ()
-  "Go to beginning of previous Roxygen entry. "
+  "Go to beginning of previous Roxygen entry."
   (interactive)
   (if (ess-roxy-entry-p)
       (progn
@@ -836,7 +738,7 @@ See `hs-show-block' and `ess-roxy-hide-block'."
   (goto-char (ess-roxy-beg-of-entry)))
 
 (defun ess-roxy-next-entry ()
-  "Go to beginning of next Roxygen entry. "
+  "Go to beginning of next Roxygen entry."
   (interactive)
   (if (ess-roxy-entry-p)
       (progn
@@ -846,8 +748,7 @@ See `hs-show-block' and `ess-roxy-hide-block'."
   (goto-char (ess-roxy-beg-of-entry)))
 
 (defun ess-roxy-get-function-args ()
-  "Return the arguments specified for the current function as a
-list of strings."
+  "Return the arguments specified for the current function as a list of strings."
   (save-excursion
     (let ((args-txt
            (progn
@@ -866,20 +767,22 @@ list of strings."
       (split-string args-txt ","))))
 
 (defun ess-roxy-match-paren ()
-  "Go to the matching parenthesis"
+  "Go to the matching parenthesis."
   (cond ((looking-at "\\s\(") (forward-list 1) (backward-char 1))
         ((looking-at "\\s\)") (forward-char 1) (backward-list 1))))
 
 (defun ess-roxy-complete-tag ()
-  "complete the tag at point"
-  (let ((token-string (thing-at-point 'symbol)))
-    (when (and token-string (string-match "@.+" token-string))
-      (comint-dynamic-simple-complete
-       (replace-regexp-in-string "^@" "" token-string)
-       (append ess-roxy-tags-noparam ess-roxy-tags-param)))))
+  "Complete the tag at point."
+  (let ((bounds (ess-bounds-of-symbol)))
+    (when (and bounds
+               (save-excursion
+                 (goto-char (car bounds))
+                 (eq (following-char) ?@)))
+      (list (1+ (car bounds)) (cdr bounds)
+            (append ess-roxy-tags-noparam ess-roxy-tags-param)))))
 
 (defun ess-roxy-tag-completion ()
-  "Completion data for emacs >= 24"
+  "Completion data for Emacs >= 24."
   (when (save-excursion (re-search-backward "@\\<\\(\\w*\\)" (point-at-bol) t))
     (let ((token (match-string-no-properties 1))
           (beg (match-beginning 1))
@@ -888,9 +791,9 @@ list of strings."
         (list beg end (append ess-roxy-tags-noparam ess-roxy-tags-param) :exclusive 'no)))))
 
 (defun ess-roxy-remove-roxy-re (string)
-  "Remove the `ess-roxy-str' before sending to R process. Useful
-  for sending code from example section.  This function is placed
-  in `ess-presend-filter-functions'."
+  "Remove `ess-roxy-str' from STRING before sending to R process.
+Useful for sending code from example section. This function is
+placed in `ess-presend-filter-functions'."
   ;; Only strip the prefix in the @examples field, and only when
   ;; STRING is entirely contained inside it. This allows better
   ;; behaviour for evaluation of regions.
@@ -898,7 +801,7 @@ list of strings."
            (with-temp-buffer
              (insert string)
              (ess-roxy-entry-p)))
-      (replace-regexp-in-string ess-roxy-re "" string)
+      (replace-regexp-in-string (concat ess-roxy-re "\\s-*") "" string)
     string))
 (add-hook 'ess-presend-filter-functions 'ess-roxy-remove-roxy-re nil)
 
@@ -940,14 +843,14 @@ list of strings."
          ,@body))))
 
 (defadvice ess-eval-line-and-step (around ess-eval-line-and-step-roxy)
-  "evaluate line but do not skip over comment (roxy) lines"
+  "Evaluate line but do not skip over comment (roxy) lines."
   (if (ess-roxy-entry-p)
       (let ((simple-next t))
         ad-do-it)
     ad-do-it))
 
 (defadvice ess-indent-command (around ess-roxy-toggle-hiding)
-  "hide this block if we are at the beginning of the line"
+  "Hide this block if we are at the beginning of the line."
   (if (and (= (point) (point-at-bol)) (ess-roxy-entry-p) 'ess-roxy-hide-show-p)
       (progn (ess-roxy-toggle-hiding))
     ad-do-it))
@@ -967,22 +870,22 @@ list of strings."
     (ess-roxy-with-filling-context t
       ad-do-it))
    ((and (not (ess-roxy-entry-p))
-         (ess-within-comment-p))
+         (ess-inside-comment-p))
     ad-do-it)
    ;; Filling of call arguments with point on call name
    ((and ess-fill-calls
-         (ess-within-call-name-p))
+         (ess-inside-call-name-p))
     (save-excursion
       (skip-chars-forward "^([")
       (forward-char)
       (ess-fill-args)))
    ;; Filling of continuations
    ((and ess-fill-continuations
-         (ess-within-continuation-p))
+         (ess-inside-continuation-p))
     (ess-fill-continuations))
    ;; Filling of call arguments
    ((and ess-fill-calls
-         (ess-within-call-p))
+         (ess-inside-call-p))
     (ess-fill-args))
    ;; Filling of roxy blocks
    ((ess-roxy-entry-p)
@@ -1018,7 +921,7 @@ list of strings."
     ad-do-it)))
 
 (defadvice move-beginning-of-line (around ess-roxy-beginning-of-line)
-  "move to start"
+  "Move to start."
   (if (ess-roxy-entry-p)
       (let ((new-pos (save-excursion
                        (end-of-line)
@@ -1031,7 +934,7 @@ list of strings."
     ad-do-it))
 
 (defadvice back-to-indentation (around ess-roxy-back-to-indentation)
-  "Handle back-to-indentation in roxygen doc"
+  "Handle `back-to-indentation' in roxygen doc."
   (if (ess-roxy-entry-p)
       (progn
         (end-of-line)
@@ -1069,21 +972,4 @@ list of strings."
 
 (provide 'ess-roxy)
 
- ; Local variables section
-
-;;; This file is automatically placed in Outline minor mode.
-;;; The file is structured as follows:
-;;; Chapters:     ^L ;
-;;; Sections:    ;;*;;
-;;; Subsections: ;;;*;;;
-;;; Components:  defuns, defvars, defconsts
-;;;              Random code beginning with a ;;;;* comment
-
-;;; Local variables:
-;;; mode: emacs-lisp
-;;; outline-minor-mode: nil
-;;; mode: outline-minor
-;;; outline-regexp: "\^L\\|\\`;\\|;;\\*\\|;;;\\*\\|(def[cvu]\\|(setq\\|;;;;\\*"
-;;; End:
-
 ;;; ess-roxy.el ends here
