@@ -25,7 +25,7 @@
 ;; GNU General Public License for more details.
 
 ;; A copy of the GNU General Public License is available at
-;; http://www.r-project.org/Licenses/
+;; https://www.r-project.org/Licenses/
 
 ;;; Commentary:
 
@@ -37,12 +37,11 @@
 ;;; Requires and Autoloads:
 
 (require 'ess-s-lang)
+(require 'ess-trns)
+(defvar ess-S+-startup-delay)
+(defvar version-function-name)
 ;;NO: this is autoloaded from other places (require 'ess-dde)
 
-(autoload 'inferior-ess "ess-inf" "Run an ESS process.")
-(autoload 'ess-mode     "ess-mode" "Edit an ESS process.")
-
-
 (defvaralias 'inferior-S+6-multipleinstances 'inferior-S+-multipleinstances)
 (defvar inferior-S+-multipleinstances "/MULTIPLEINSTANCES"
   "Default \"/MULTIPLEINSTANCES\" opens up a new instance of S+[678] in a
@@ -55,10 +54,9 @@ connects it to the '(ddeESS [S+])' window.")
   (append
    '((ess-local-customize-alist  . 'S+-customize-alist)
      (ess-dialect                . S+-dialect-name)
-     (ess-loop-timeout          . ess-S-loop-timeout);fixme: dialect spec.
      (ess-object-name-db-file    . "ess-sp6-namedb.el" )
      (ess-display-help-on-object-function . #'ess-chm-display-help-on-object)
-     (inferior-ess-program       . inferior-S+-program-name)
+     (inferior-ess-program       . inferior-S+-program)
      (inferior-ess-help-command  . "help(\"%s\")\n")
      (inferior-ess-help-filetype . "chm")
      (inferior-ess-search-list-command . "searchPaths()\n")
@@ -70,7 +68,10 @@ connects it to the '(ddeESS [S+])' window.")
                                     " "
                                     inferior-S+-print-command
                                     " S_PROJ="
-                                    (w32-short-file-name (directory-file-name default-directory))))
+                                    (when (fboundp 'w32-short-file-name)
+                                      (w32-short-file-name
+                                       (directory-file-name
+                                        default-directory)))))
      ;;    (inferior-ess-ddeclient      . "ddeclient")
      ;;    (inferior-ess-client-name    . "S-PLUS")
      ;;    (inferior-ess-client-command . "SCommand")
@@ -87,7 +88,7 @@ connects it to the '(ddeESS [S+])' window.")
      (ess-loop-timeout           . 500000 );fixme: dialect specific custom.var
      (ess-object-name-db-file    . "ess-sp6-namedb.el" )
      (ess-display-help-on-object-function . #'ess-chm-display-help-on-object)
-     (inferior-ess-program       . inferior-Sqpe+-program-name)
+     (inferior-ess-program       . inferior-Sqpe+-program)
      (inferior-ess-help-command  . "help(\"%s\")\n")
      (inferior-ess-help-filetype . "chm")
      (inferior-ess-search-list-command . "searchPaths()\n")
@@ -134,23 +135,23 @@ connects it to the '(ddeESS [S+])' window.")
 ;;;
 (defalias 'S+6 'S+)
 (defun S+ (&optional proc-name)
-  "Verify that `inferior-S+-program-name' points to S-Plus 6 or
+  "Verify that `inferior-S+-program' points to S-Plus 6 or
 S-Plus 7 or S-Plus 8.  Start normally for S-Plus 6.1 and later.
 Inform the user to start S-Plus 6.0 from the icon and then
 connect to it with `S+-existing'.  Give an error message if
-`inferior-S+-program-name' doesn't point to S-Plus 6 or S-Plus 7
+`inferior-S+-program' doesn't point to S-Plus 6 or S-Plus 7
 or S-Plus 8."
   (interactive)
   (with-current-buffer (find-file-noselect
-                        (concat (executable-find inferior-S+-program-name)
+                        (concat (executable-find inferior-S+-program)
                                 "/../../versions") t)
     (setq buffer-read-only 1)
     (forward-line)
     (if (not (search-backward-regexp "splus\t[678].[0-9]" (point-min) t))
-        (error "The emacs variable `inferior-S+-program-name' does
+        (error "The emacs variable `inferior-S+-program' does
 not point to S-Plus 6 or 7 or 8.  Please add `splus[678]?/cmd' (expand the
 `[678]?' to match your setup) to your `exec-path' or specify the complete
-path to `Splus.exe' in the variable `inferior-S+-program-name' in your
+path to `Splus.exe' in the variable `inferior-S+-program' in your
 `.emacs' file.")
       (forward-line)
       (if (search-backward "splus\t6.0" (point-min) t)
@@ -189,8 +190,11 @@ to start the Splus program."
     (let ((s-proj (getenv "S_PROJ"))
           (use-dialog-box (not (or ess-microsoft-p (eq system-type 'cygwin))))
           )
-      (cd (w32-short-file-name (directory-file-name default-directory)))
-      (setenv "S_PROJ" (w32-short-file-name default-directory))
+      (cd (when (fboundp 'w32-short-file-name)
+            (w32-short-file-name (directory-file-name default-directory))))
+      (setenv "S_PROJ" (when
+                           (fboundp 'w32-short-file-name)
+                         (w32-short-file-name default-directory)))
       (inferior-ess)
       (sleep-for 2) ; need to wait, else working too fast!  The Splus
                                         ; command in '(ddeESS [S+])' should follow the "$"
@@ -207,7 +211,7 @@ to start the Splus program."
     (setq comint-process-echoes nil)
     (setq comint-input-sender 'comint-simple-send)
     (goto-char (point-max))
-    (insert (concat inferior-S+-program-name " "
+    (insert (concat inferior-S+-program " "
                     inferior-ess-start-args)) ; Note: there is no final "&".
     ;; Without the "&", the results of  !system.command  come to '(ddeESS [S+])'
     ;; With the "&", the results of  !system.command  in S get lost.
@@ -230,7 +234,8 @@ Any results of the   !system.command   typed at the S prompt in the
 Splus Commands window appear in this buffer.\n\n")
     (goto-char (point-max))             ; comint-mode-map makes '(ddeESS [S+])'
     ;;  (use-local-map comint-mode-map)     ;a shell buffer after Splus is finished.
-    (set-buffer-process-coding-system 'raw-text-dos 'raw-text-unix)
+    (set-process-coding-system (get-buffer-process (current-buffer))
+                               'raw-text-dos 'raw-text-unix)
     (setq buffer-read-only t)           ; force buffer to be read-only
     (setq mode-name "ddeESS")
     ;;  (ess-eval-linewise inferior-S+-editor-pager-command)
@@ -317,6 +322,14 @@ Splus Commands window blink a DOS window and you won't see them.\n\n")
         (ess-eval-linewise inferior-ess-language-start))
     (if shome-nil-p (setenv "SHOME" nil))))
 
+;;;###autoload
+(add-to-list 'auto-mode-alist '("\\.sp\\'" . S-mode))
+;;;###autoload
+(add-to-list 'auto-mode-alist '("\\.[qsS]\\'" . S-mode))
+;;;###autoload
+(add-to-list 'auto-mode-alist '("\\.ssc\\'" . S-mode))
+;;;###autoload
+(add-to-list 'auto-mode-alist '("\\.SSC\\'" . S-mode))
 
 (defalias 'S+6-mode 'S+-mode)
 (defun S+-mode (&optional proc-name)
@@ -332,26 +345,25 @@ Splus Commands window blink a DOS window and you won't see them.\n\n")
   (interactive)
   (ess-transcript-mode S+-customize-alist))
 
-(defalias 'S+-msdos 'S+-msdos)
 (defun S+-msdos (&optional proc-name)
-  "Verify that `inferior-S+-program-name' points to S-Plus 6 or
+  "Verify that `inferior-S+-program' points to S-Plus 6 or
 S-Plus 7 or S-Plus 8.  Start normally for S-Plus 6.1 and later.
 Inform the user to start S-Plus 6.0 from the icon and then
 connect to it with `S+-msdos-existing'.  Give an error message
-if `inferior-S+-program-name' doesn't point to S-Plus 6 or
+if `inferior-S+-program' doesn't point to S-Plus 6 or
 S-Plus 7 or S-Plus 8."
   (interactive)
   (with-current-buffer  (find-file-noselect
-                 (concat (executable-find inferior-S+-program-name)
-                         "/../../versions") t)
+                         (concat (executable-find inferior-S+-program)
+                                 "/../../versions") t)
     (setq buffer-read-only 1)
     (forward-line)
     (if (not (search-backward-regexp "splus\t[678].[0-9]" (point-min) t))
-        (error "The emacs variable `inferior-S+-program-name' does
+        (error "The emacs variable `inferior-S+-program' does
  not point to S-Plus 6 or 7 or 8.  Please add `splus[678]?/cmd'
  (expand the `[678]?' to match your setup) to your `exec-path' or
  specify the complete path to `Splus.exe' in the variable
-`inferior-S+-program-name' in your `.emacs' file.")  ;;; " This comment keeps emacs font-lock from getting out of phase.
+`inferior-S+-program' in your `.emacs' file.")  ;;; " This comment keeps emacs font-lock from getting out of phase.
 
       (progn
         (forward-line)
@@ -393,8 +405,11 @@ to start the Splus program."
     (let ((s-proj (getenv "S_PROJ"))
           (use-dialog-box (not (or ess-microsoft-p (eq system-type 'cygwin))))
           )
-      (cd (w32-short-file-name (directory-file-name default-directory)))
-      (setenv "S_PROJ" (w32-short-file-name default-directory))
+      (cd (when (fboundp 'w32-short-file-name)
+            (w32-short-file-name (directory-file-name default-directory))))
+      (setenv "S_PROJ" (when
+                           (fboundp 'w32-short-file-name)
+                         (w32-short-file-name default-directory)))
       (inferior-ess)
       (sleep-for 2) ; need to wait, else working too fast!  The Splus
                                         ; command in '(ddeESS [S+])' should follow the "$"
@@ -410,9 +425,10 @@ to start the Splus program."
 ;;; end of what belongs in customize-alist
     (setq comint-input-sender 'comint-simple-send)
     (setq comint-process-echoes nil)
-    (set-buffer-process-coding-system 'raw-text-dos 'raw-text-dos)
+    (set-process-coding-system (get-buffer-process (current-buffer))
+                               'raw-text-dos 'raw-text-dos)
     (goto-char (point-max))
-    (insert (concat inferior-S+-program-name " "
+    (insert (concat inferior-S+-program " "
                     inferior-ess-start-args)) ; Note: there is no final "&".
     ;; Without the "&", the results of  !system.command  come to '(ddeESS [S+])'
     ;; With the "&", the results of  !system.command  in S get lost.
@@ -447,9 +463,9 @@ Splus Commands window (are supposed to) appear in this buffer.\n\n")
     (use-local-map comint-mode-map)    ; a shell buffer after Splus is finished.
     (setq buffer-read-only t)          ; force buffer to be read-only
     (setq mode-name "ddeESS")
-;;  (ess-eval-linewise inferior-S+-editor-pager-command)
+    ;;  (ess-eval-linewise inferior-S+-editor-pager-command)
     (if inferior-ess-language-start
-      (ess-eval-linewise inferior-ess-language-start))
+        (ess-eval-linewise inferior-ess-language-start))
     ))
 
 (defalias 'S+6-msdos-existing 'S+-msdos-existing)
@@ -487,7 +503,7 @@ Sqpe.  `ESS-SHOME-VERSIONS' is normally taken from
 functions are created.  This works by creating a temp buffer where the
 template function `Sqpe+template' is edited by replacing the string
 'Sqpe+template' by the version name.  The list of functions actually
-created appears in the *ESS* buffer.  If `X64' is not nil, then
+created appears in the `ess-dribble-buffer'.  If `X64' is not nil, then
 modify the function name to show \"-64bit\" in its name.
 
 The result `ess-sqpe-versions-created' will store a list of the new
@@ -548,7 +564,7 @@ This function was generated by `ess-sqpe-versions-create'."
   (let* ((use-dialog-box) ;; MS dialog box won't return a directory
          (shome-old (getenv "SHOME"))
          (inferior-Sqpe+-SHOME-name "ess-SHOME")
-         (inferior-Sqpe+-program-name (concat "ess-SHOME" "/cmd/sqpe.exe")))
+         (inferior-Sqpe+-program (concat "ess-SHOME" "/cmd/sqpe.exe")))
     (setenv "SHOME" "ess-SHOME")
     (ess-write-to-dribble-buffer
      (format "\n(Sqpe+template): ess-dialect=%s, buf=%s\n" ess-dialect
@@ -584,21 +600,4 @@ This function was generated by `ess-sqpe-versions-create'."
 
 (provide 'ess-sp6w-d)
 
- ; Local variables section
-
-;;; This file is automatically placed in Outline minor mode.
-;;; The file is structured as follows:
-;;; Chapters:     ^L ;
-;;; Sections:    ;;*;;
-;;; Subsections: ;;;*;;;
-;;; Components:  defuns, defvars, defconsts
-;;;              Random code beginning with a ;;;;* comment
-
-;;; Local variables:
-;;; mode: emacs-lisp
-;;; outline-minor-mode: nil
-;;; mode: outline-minor
-;;; outline-regexp: "\^L\\|\\`;\\|;;\\*\\|;;;\\*\\|(def[cvu]\\|(setq\\|;;;;\\*"
-;;; End:
-
 ;;; ess-sp6w-d.el ends here
