@@ -24,7 +24,7 @@
 ;; GNU General Public License for more details.
 
 ;; A copy of the GNU General Public License is available at
-;; http://www.r-project.org/Licenses/
+;; https://www.r-project.org/Licenses/
 
 ;;; Commentary:
 
@@ -50,9 +50,11 @@
 (require 'make-regexp)  ; it's now local to the directory.
 ;;(load-library "make-regexp") ;; this is necessary for
 ;; ado-set-font-lock-keywords
-;; only needed in Emacs >= 22.x and newish Xemacsen:
+;; only needed in Emacs >= 22.x
 (unless (boundp 'c-emacs-features)
   (require 'cc-vars));; for syntax-table
+(require 'comint)
+(require 'ess-trns)
 
                                         ;(setq max-lisp-eval-depth 500)
 (eval-when-compile
@@ -1277,21 +1279,4 @@ Active commands are Help (\\[stata-help]) and hyperlink
 
 (provide 'ess-stata-lang)
 
- ; Local variables section
-
-;;; This file is automatically placed in Outline minor mode.
-;;; The file is structured as follows:
-;;; Chapters:     ^L ;
-;;; Sections:    ;;*;;
-;;; Subsections: ;;;*;;;
-;;; Components:  defuns, defvars, defconsts
-;;;              Random code beginning with a ;;;;* comment
-
-;;; Local variables:
-;;; mode: emacs-lisp
-;;; outline-minor-mode: nil
-;;; mode: outline-minor
-;;; outline-regexp: "\^L\\|\\`;\\|;;\\*\\|;;;\\*\\|(def[cvu]\\|(setq\\|;;;;\\*"
-;;; End:
-
 ;;; ess-stata-lang.el ends here
