@@ -24,7 +24,7 @@
 ;; GNU General Public License for more details.
 
 ;; A copy of the GNU General Public License is available at
-;; http://www.r-project.org/Licenses/
+;; https://www.r-project.org/Licenses/
 
 ;;; Commentary:
 
@@ -32,22 +32,41 @@
 
 ;;; Code:
 
+(require 'ess-arc-d)
+(require 'ess-julia)
+(require 'ess-omg-d)
 (require 'ess-s-lang)
+(require 'ess-s3-d)
+(require 'ess-s4-d)
+(require 'ess-sas-d)
+(require 'ess-sp3-d)
+(require 'ess-sp4-d)
+(require 'ess-sp5-d)
+(require 'ess-sp6-d)
+(require 'ess-sp6w-d)
+(require 'ess-stata-mode)
+(require 'ess-trns)
 (require 'ess-utils)
+(require 'ess-vst-d)
+(require 'ess-xls-d)
 
-(autoload 'inferior-ess "ess-inf" "Run an ESS process.")
-(autoload 'ess-mode     "ess-mode" "Edit an ESS process.")
 (defvar S+elsewhere-dialect-name "S+6"
   "Name of 'dialect' for S-PLUS at another location.")
                                         ;easily changeable in a user's .emacs
 
+(defcustom inferior-ess-remote-pager nil
+  "Remote pager to use for reporting help files and similar things.
+The default value is nil."
+  :group 'ess-proc
+  :type '(choice (const nil) string))
+
+
 (defvar S+elsewhere-customize-alist
   (append
    '((ess-local-customize-alist         . 'S+elsewhere-customize-alist)
      (ess-dialect                       . S+elsewhere-dialect-name)
-     (ess-loop-timeout                  . ess-S-loop-timeout);fixme: dialect spec.
      (ess-object-name-db-file           . "ess-spelsewhere-namedb.el" )
-     (inferior-ess-program              . inferior-S-elsewhere-program-name)
+     (inferior-ess-program              . inferior-S-elsewhere-program)
      (inferior-ess-help-command         . "help(\"%s\", pager=\"cat\", window=F)\n")
 
      (inferior-ess-start-file           . nil) ;"~/.ess-S+3")
@@ -57,10 +76,8 @@
    S+common-cust-alist)
   "Variables to customize for S+elsewhere")
 
-
 (defun S+elsewhere (&optional proc-name)
-  "Call 'S-PLUS 3.x', the 'Real Thing'  from StatSci.
-This command is obsolete; please use `ess-remote' instead."
+  "Call 'S-PLUS 3.x', the 'Real Thing'  from StatSci."
   (interactive)
   (setq ess-customize-alist S+elsewhere-customize-alist)
   (ess-write-to-dribble-buffer
@@ -69,7 +86,8 @@ This command is obsolete; please use `ess-remote' instead."
   (inferior-ess)
   (if inferior-ess-language-start
       (ess-eval-linewise inferior-ess-language-start)))
-
+;; git commit 104c4d7c56bc239ea245562763caa317bc3a1a84
+(make-obsolete 'S+elsewhere #'ess-remote "2000")
 
 (defun S+elsewhere-mode (&optional proc-name)
   "Major mode for editing S+3 source.  See `ess-mode' for more help."
@@ -85,20 +103,6 @@ This command is obsolete; please use `ess-remote' instead."
 ;; This REALLY shouldn't need an editing mode.  Just a transcript and
 ;; an inferior process handler.
 
-(defun ess-change-alist (item value alist)
-  "Modify ALIST to set VALUE to ITEM.
-If there is a pair whose car is ITEM, replace its cdr by VALUE.
-If there is not such pair, create new pair (ITEM . VALUE) and
-return new alist whose car is the new pair and cdr is ALIST.
-\[tomo's ELIS like function]"
-  (let ((pair (assoc item alist)))
-    (if pair
-        (progn
-          (setcdr pair value)
-          alist)
-      (cons (cons item value) alist))))
-
-
 (defun ess-select-alist-dialect (&optional dialect)
   "Query user for an ESS dialect and return the matching customize-alist."
   (interactive)
@@ -126,26 +130,6 @@ return new alist whose car is the new pair and cdr is ALIST.
      (t                         S+elsewhere-customize-alist)
      )))
 
-
-;; (defun ESS-elsewhere (&optional proc-name)
-;;   "Call an inferior process from ELSEWHERE.
-;; This command is obsolete; please use `ess-remote' instead."
-;;   (interactive)
-;;   ;; Need to select a elsewhere-customize-alist
-;;   (let ((elsewhere-customize-alist (ess-select-alist-dialect)))
-;;     (ess-change-alist 'inferior-ess-program
-;;                       inferior-ESS-elsewhere-program-name
-;;                       elsewhere-customize-alist)
-;;     (setq ess-customize-alist elsewhere-customize-alist)
-;;     (ess-write-to-dribble-buffer
-;;      (format "\n(ESS-elsewhere): ess-dialect=%s, buf=%s\n" ess-dialect
-;;              (current-buffer)))
-;;     (inferior-ess)
-;;     (if (equal ess-language "S")
-;;         (if inferior-ess-language-start
-;;             (ess-eval-linewise inferior-ess-language-start)))))
-
-
 (defun ess-add-ess-process ()
   "Execute this command from within a buffer running a process to add
 the process to `ess-process-name-alist' and to make it the
@@ -164,6 +148,7 @@ buffer on the local computer."
 (defvar ess-remote nil
   "Indicator, t in ess-remote buffers.")
 
+;;;###autoload
 (defun ess-remote (&optional proc-name dialect)
   "Execute this command from within a buffer running a process.  It
 runs `ess-add-ess-process' to add the process to
@@ -198,7 +183,8 @@ DIALECT is the desired ess-dialect. If nil, ask for dialect"
 
     (when (equal ess-dialect "R")
       ;; ugly fix for evn variable. What can we do :(
-      (ess-eval-linewise (format "options(pager='%s')\n" inferior-ess-pager)
+      (ess-eval-linewise (format "options(pager='%s')\n"
+                                 (or inferior-ess-remote-pager inferior-ess-pager))
                          nil nil nil 'wait)
       (inferior-ess-r-load-ESSR))
 
@@ -212,9 +198,6 @@ DIALECT is the desired ess-dialect. If nil, ask for dialect"
       (setq buffer-read-only nil)
       (font-lock-mode 1))
 
-    (ess-process-put 'funargs-cache (make-hash-table :test 'equal))
-    (ess-process-put 'funargs-pre-cache nil)
-    (ess-process-put 'accum-buffer-name (format " *%s:accum*" ess-local-process-name))
     (ess-load-extras)
 
     (when inferior-ess-language-start
@@ -226,21 +209,4 @@ DIALECT is the desired ess-dialect. If nil, ask for dialect"
 
 (provide 'essd-els)
 
- ; Local variables section
-
-;;; This file is automatically placed in Outline minor mode.
-;;; The file is structured as follows:
-;;; Chapters:     ^L ;
-;;; Sections:    ;;*;;
-;;; Subsections: ;;;*;;;
-;;; Components:  defuns, defvars, defconsts
-;;;              Random code beginning with a ;;;;* comment
-
-;;; Local variables:
-;;; mode: emacs-lisp
-;;; outline-minor-mode: nil
-;;; mode: outline-minor
-;;; outline-regexp: "\^L\\|\\`;\\|;;\\*\\|;;;\\*\\|(def[cvu]\\|(setq\\|;;;;\\*"
-;;; End:
-
 ;;; essd-els.el ends here
